@@ -55,6 +55,12 @@
  * -change init function name to be more in line with de facto standards
  * -change malloc/free to ckalloc/ckfree in case TCL_MEM_DEBUG set
  *-----------------------------------------------------------------------------
+ * Version 2.1, February, 1994
+ * Tom Poindexter
+ * tpoind@advtech.uswest.com  or  tpoindex@nyx.cs.du.edu
+ * -add commands option to sybnext, hacked from Oratcl 2.1
+ * -change Sybtcl_Init to (int)
+ *-----------------------------------------------------------------------------
  *
  */
 
@@ -475,7 +481,7 @@ Tcl_KillSyb (clientData)
  *   Tcl_CreateExtendedInterp.
  */
 
-void
+int
 Sybtcl_Init (interp)
     Tcl_Interp *interp;
 {
@@ -546,6 +552,8 @@ Sybtcl_Init (interp)
     /* other indices - correspond to error and message handler arguments */
     clear_msg(interp);
 
+    return TCL_OK;
+
 }
 
 
@@ -798,7 +806,7 @@ Tcl_SybSql (clientData, interp, argc, argv)
  *
  * Tcl_SybNext --
  *    Implements the sybnext command:
- *    usage: sybnext handle 
+ *    usage: sybnext handle ?commands?
  *	                
  *    results:
  *	next row from latest results as tcl list, or null list
@@ -822,12 +830,18 @@ Tcl_SybNext (clientData, interp, argc, argv)
     int     col_len;
     BYTE   *col_ptr;
     int     dst_len;
+    int     Tcl_SybNextAll();
 
 
     if ((hand = syb_prologue(interp, argc, argv, 2, " handle")) == -1) {
 	return TCL_ERROR;
     }
 
+    /* if commands argument, then call next function to do work */
+    if (argc >= 3) {
+	return (Tcl_SybNextAll(clientData, interp, argc, argv));
+    }
+
     /* check to see if next set of results */
 
     if (SybProcs[hand].last_next == NO_MORE_ROWS) {
@@ -952,6 +966,239 @@ Tcl_SybNext (clientData, interp, argc, argv)
 
 
 
+/*
+ *----------------------------------------------------------------------
+ *
+ * Tcl_SybNextAll --
+ *    Implements the sybnext with iterative commands:
+ *    hacked from Oratcl 2.1, nearly verbatim
+ *    usage: sybnext cur_handle tcl_stmts
+ *	                
+ *    results:
+ *	fetch all rows from existing query, execute tcl_stmt on each row
+ *	sets message array element "rc" with value of final fetch rcode
+ *      TCL_OK - handle is opened
+ *      TCL_ERROR - wrong # args, or handle not opened, tcl_stmt failed,
+ *                  or wrong number of columns
+ */
+
+static int
+Tcl_SybNextAll (clientData, interp, argc, argv)
+    ClientData   clientData;
+    Tcl_Interp  *interp;
+    int          argc;
+    char       **argv;
+{
+    int      hand;
+    char     buf[SYB_BUFF_SIZE];
+    char    *eval_ptr;
+    int      colsCnt;
+    char   **colsPtr;
+    int      i;
+    char    *s;
+    char    *s2;
+    char    *p;
+    int      max_col = 0;
+
+#define NUMSTRS 300		/* number of individual strings to allow*/
+#define SUBCHAR '@'		/* substitution character */
+
+    int      num_str = 0;	/* number of strings to concat */
+    char    *str[NUMSTRS];      /* strings to pass to concat   */
+
+    int      colnum  = 0; 	/* number of colstr's used */
+
+    struct {			
+      int      column; 		/* column number index */
+      int      strnum;          /* str array index */
+    } colstr[NUMSTRS];
+
+    Tcl_DString evalStr;	/* tcl dynamic string for building eval */
+    int        inum;
+    int        icol;
+    int        tcl_rc;
+    int        id;
+
+
+    if ((hand = syb_prologue(interp, argc, argv, 3, 
+			    " cur_handle  commands")) == -1) {
+	return TCL_ERROR;
+    }
+
+    /* check if already exahusted or nothing to fetch */
+
+    if (SybProcs[hand].last_results == NO_MORE_RESULTS) {
+	/* sybnext return  NO_MORE_RESULTS last time, tell'm again! */
+	Tcl_SetVar2(interp, SybMsgArray, "nextrow","NO_MORE_RESULTS", 
+			TCL_GLOBAL_ONLY);
+        SybProcs[hand].last_next = NO_MORE_ROWS;
+
+	return TCL_OK;
+    }
+
+    /* parse tcl_stmt for all '@' substitutions */
+
+    s  = argv[2];			/* start of tcl_stmt */
+    s2 = s;
+
+    while ( (p = strchr(s2,SUBCHAR)) != NULL) {
+
+      if (isdigit(*(p+1))) {		/* it's a substituion value ! */
+
+	  if (s != p) {			/* subchar not at the front of string?*/
+	      str[num_str] = s;		/* save the current front of string */
+	      *p = '\0';  		/* terminate the literal string     */
+	      num_str++;		/* into str array and               */
+	  }
+
+          i = atoi(p + 1);		/* the column number */
+
+	  max_col = (i > max_col) ? i : max_col;/* sanity check, max col num */
+
+	  colstr[colnum].column = i;		/* save column number */
+	  colstr[colnum].strnum = num_str;	/* and str positiion  */
+	  str[num_str] = "";			/* sanity check       */
+	  colnum++;
+	  num_str++;
+	  p++;				/* skip past col number */
+	  while (isdigit(*p)) {
+	      p++;
+	  }
+	  s  = p;			/* next point to look for subchar */
+	  s2 = p;			
+
+
+      } else {				/* it's a subchar without number */
+	  s2 = p + 1;			/* ignore and keep searching     */
+      }
+
+    }
+
+    if (num_str == 0) {    		/* check if no substitutions */
+	str[num_str] = argv[2];	
+	num_str = 1;	
+    } else {		
+	if (strlen(s) > 0) {		/* remainder of tcl_stmt string */
+            str[num_str] = s;
+	    num_str++;
+	}
+    }
+
+
+
+    /* loop until fetch exhausted */
+
+    if (Tcl_SybNext(clientData, interp, 2, argv) == TCL_ERROR) {
+	Tcl_AppendResult (interp, ": ", argv[0], ": sybnext failed", 
+			  (char *) NULL);
+	return TCL_ERROR;
+    }
+    if (SybProcs[hand].last_next == NO_MORE_ROWS) {
+        if (SybProcs[hand].last_results == NO_MORE_RESULTS) {
+	    Tcl_SetVar2(interp, SybMsgArray, "nextrow","NO_MORE_RESULTS", 
+			    TCL_GLOBAL_ONLY);
+	} else {
+	    Tcl_SetVar2(interp, SybMsgArray, "nextrow","NO_MORE_ROWS", 
+			    TCL_GLOBAL_ONLY);
+	}
+	return TCL_OK;
+    }
+
+    /* make sure there are enough columns in result */
+    i = 0;
+
+    /* check for a regular row */
+    if (SybProcs[hand].last_next == REG_ROW) {
+	i = dbnumcols(SybProcs[hand].dbproc);
+    } else {
+	id = SybProcs[hand].last_next;  /* the compute row id */
+	i = dbnumalts(SybProcs[hand].dbproc,id);
+    }
+
+    if (max_col > i) {
+	Tcl_AppendResult (interp, argv[0], ": @column number execeeds results", 
+			  (char *) NULL);
+	return TCL_ERROR;
+    }
+
+    while (SybProcs[hand].last_next != NO_MORE_ROWS) {
+
+        /* split the result columns left in interp->result */
+	if (Tcl_SplitList(interp,interp->result,&colsCnt,&colsPtr) == 
+							    TCL_ERROR) {
+	   Tcl_AppendResult (interp, argv[0], ": split columns failed", 
+			      (char *) NULL);
+	   return TCL_ERROR;
+	}
+
+        /* build eval string from literals and columns */
+        Tcl_DStringInit(&evalStr);
+	eval_ptr = "";
+	for (inum=0, icol=0; inum < num_str; inum++) {
+	    if (inum == colstr[icol].strnum) {
+		eval_ptr = Tcl_DStringAppend(&evalStr, " ", -1);
+		if (colstr[icol].column == 0) {    /* col 0 is entire result */
+		    eval_ptr = Tcl_DStringAppendElement(&evalStr,
+							interp->result);
+                } else {
+		    if (colstr[icol].column > colsCnt) {/* another sanity chk*/
+			Tcl_AppendResult (interp, argv[0], 
+			       ": column sanity failed!",(char *) NULL);
+	                Tcl_DStringFree(&evalStr); 	/* free eval string */
+	                ckfree((char *) colsPtr); 	/* free split array */
+			return TCL_ERROR;
+		    }
+		    eval_ptr = Tcl_DStringAppendElement(&evalStr,
+					  colsPtr[colstr[icol].column - 1]);
+		}
+		eval_ptr = Tcl_DStringAppend(&evalStr, " ", -1);
+	        icol++;
+	    } else {
+	        eval_ptr = Tcl_DStringAppend(&evalStr, str[inum], -1);
+	    }
+	}
+
+        tcl_rc = Tcl_Eval(interp, eval_ptr);	/* do it! */
+
+	Tcl_DStringFree(&evalStr); 	/* free eval string */
+
+	ckfree((char *) colsPtr); 	/* free split array */
+
+	switch (tcl_rc) {		/* check on eval return code */
+	  case TCL_ERROR:
+	    Tcl_AppendResult (interp, ": eval failed while in ", argv[0], 
+			      (char *) NULL);
+	    return TCL_ERROR;
+	    break;
+
+	  case TCL_BREAK:		/* return sooner */
+	    return TCL_OK;
+	    break;
+
+	  default:
+	    break;
+	}
+
+	Tcl_ResetResult(interp);	/* reset interp result for next */
+ 
+        /* next fetch */
+        if (Tcl_SybNext(clientData, interp, 2, argv) == TCL_ERROR) {
+	    Tcl_AppendResult (interp, ": ", argv[0], ": sybnext failed", 
+			      (char *) NULL);
+	    return TCL_ERROR;
+        }
+
+    }
+
+    return TCL_OK;
+
+}
+
+
+
+
+
+
 /*
  *----------------------------------------------------------------------
  *
